/***************************************************************************/
/*                                                                         */
/*  ttgxvar.h                                                              */
/*                                                                         */
/*    TrueType GX Font Variation loader  (Specification)                   */
/*                                                                         */
/*  Copyright 2004 by                                                      */
/*  David Turner, Robert Wilhelm, Werner Lemberg and George Williams.      */
/*                                                                         */
/*  This file is part of the FreeType project, and may only be used,       */
/*  modified, and distributed under the terms of the FreeType project      */
/*  license, LICENSE.TXT.  By continuing to use, modify, or distribute     */
/*  this file you indicate that you have read the license and              */
/*  understand and accept it fully.                                        */
/*                                                                         */
/***************************************************************************/
#ifdef TT_CONFIG_OPTION_GX_VAR_SUPPORT


#ifndef __TTGXVAR_H__
#define __TTGXVAR_H__


#include <ft2build.h>
#include "ttobjs.h"


FT_BEGIN_HEADER


  /*************************************************************************/
  /*                                                                       */
  /* <Struct>                                                              */
  /*    GX_AVarRec                                                         */
  /*                                                                       */
  /* <Description>                                                         */
  /*    Data from the 'avar' table.                                        */
  /*    There's one of these for each axis                                 */
  /*                                                                       */
  /* <Fields>                                                              */
  /*************************************************************************/

  typedef struct  GX_AVarRec_
  {
    FT_Int    num_points;
    FT_Fixed *orig;             /* array with num_points entries */
    FT_Fixed *final;            /* array with num_points entries */
  } GX_AVarRec, *GX_AVar;



  /*************************************************************************/
  /*                                                                       */
  /* <Struct>                                                              */
  /*    GX_BlendRec                                                        */
  /*                                                                       */
  /* <Description>                                                         */
  /*    Data for interpolating a font from a distortable font specified    */
  /*    by the GX *var tables ([fgca]var).                                 */
  /*                                                                       */
  /* <Fields>                                                              */
  /*    num_axis         :: The number of axes along which interpolation   */
  /*                         may happen                                    */
  /*    normalizedcoords :: A normalized value (between [-1,1]) indicating */
  /*                         the contribution along each axis to the final */
  /*                         interpolated font.                            */
  /*************************************************************************/

  typedef struct  GS_BlendRec_
  {
    FT_UInt          num_axis;
    FT_Fixed        *normalizedcoords;

    FT_MM_Var       *mmvar;
    FT_Int           mmvar_len;

    FT_Bool          avar_checked;
    GX_AVar          avar;

    FT_Int           tuplecount;         /* shared tuples in 'gvar' */
    FT_Fixed        *tuplecoords;        /* tuplecoords[tuplecount][axis_cnt] */

    FT_Int           gv_glyphcnt;
    FT_ULong        *glyphoffsets;
    
  } GX_BlendRec;




  FT_LOCAL_DEF( FT_Error )
  TT_Set_MM_Blend( TT_Face    face,
                   FT_UInt    num_coords,
                   FT_Fixed*  coords );

  FT_LOCAL_DEF( FT_Error )
  TT_Set_Var_Design( TT_Face   face,
                     FT_UInt   num_coords,
                     FT_Fixed* coords );

  FT_LOCAL_DEF( FT_Error )
  TT_Get_MM_Var( TT_Face      face,
                 FT_MM_Var**  master );


  FT_LOCAL_DEF( FT_Error )
  tt_face_vary_cvt( TT_Face    face,
                    FT_Stream  stream );

  FT_LOCAL_DEF( FT_Error )
  TT_Vary_Get_Glyph_Deltas( TT_Face    face,
			    FT_UInt    glyph_index,
		       	    FT_Vector  **deltas,
			    FT_UInt    n_points);

  FT_LOCAL_DEF( void )
  tt_done_blend( FT_Memory,
      GX_Blend    blend );

FT_END_HEADER

#endif /* __TTGXVAR_H__ */

#endif /* TT_CONFIG_OPTION_GX_VAR_SUPPORT */


/* END */
